import subprocess, random, os, glob, sys, signal
import numpy as np
import pandas as pd
import networkx as nx
from scipy.stats import pearsonr, lognorm
from collections import Counter, defaultdict

class TimeoutException(Exception):
   pass

def timeout_handler(signum, frame):
   raise TimeoutException

signal.signal(signal.SIGALRM, timeout_handler)

# Generate powerlaw popularity distribution for news sources
sys.stderr.write("Generating sources...\n")
os.mkdir("synth_data/run%s" % sys.argv[1])
os.chdir("synth_data/run%s" % sys.argv[1])

source_popularity = pd.read_csv("../data/popularity_distribution_fb.csv", header = None, names = ("popularity",))["popularity"].values.astype(int)

# Generate a normal polarity distribution with avg = 0, and min/max -1/1
source_polarity = np.random.normal(size = source_popularity.shape[0])
source_polarity /= np.max(np.abs(source_polarity))
print("Mean source polarity: %1.4f" % np.mean(source_polarity))
source_truthfulness = np.abs(source_polarity) + np.random.uniform(low = -0.05, high = 0.05, size = source_popularity.shape[0])
source_truthfulness = 1.0 - (source_truthfulness / np.max(np.abs(source_truthfulness)))
print("Mean source truthfulness: %1.4f" % np.mean(source_truthfulness))
r, p = pearsonr(np.abs(source_polarity), source_truthfulness)
print("polarity-truthfulness correlation: %1.4f (p = %1.4f)" % (r, p))

source_polarity_plot = Counter(np.round(source_polarity, decimals = 1))

with open("source_polarity", 'w') as f:
   for i in range(len(source_polarity)):
      f.write("%s\t%s\n" % (i, source_polarity[i]))

with open("source_truthfulness", 'w') as f:
   for i in range(len(source_truthfulness)):
      f.write("%s\t%s\n" % (i, source_truthfulness[i]))

with open("source_polarity_distribution", 'w') as f:
   for p in sorted(source_polarity_plot.keys()):
      f.write("%s\t%s\n" % (p, source_polarity_plot[p]))

# Generate the polarity of the audience, with avg = 0, and min/max -1/1
sys.stderr.write("Generating social & audience networks...\n")
number_users = int(max(source_popularity) * 2.5)
print("# users: %d" % number_users)
user_polarity = np.random.normal(size = number_users)
user_polarity /= np.max(np.abs(user_polarity))
print("Mean user polarity: %1.4f" % np.mean(user_polarity))
user_polarity_plot = Counter(np.round(user_polarity, decimals = 1))
with open("user_polarity_distribution", 'w') as f:
   for p in sorted(user_polarity_plot.keys()):
      f.write("%s\t%s\n" % (p, user_polarity_plot[p]))

# Make the social network, an LFR benchmark
G = nx.gnm_random_graph(number_users, 65459)
while nx.number_connected_components(G) > 1:
   ccomps = list(nx.connected_components(G))
   for ccomp in ccomps[1:]:
      for node in ccomp:
         G.add_edge(node, random.choice(list(ccomps[0])))

nx.set_node_attributes(G, {n: 0 for n in G.nodes}, name = "polarity")
nx.write_edgelist(G, "social_network", data = False)

# Assign user to polarity
_user_polarity = user_polarity.copy()
users_needing_polarity = set(G.nodes)
polarity_user = {}
while len(users_needing_polarity) > 0:
   node = random.sample(users_needing_polarity, 1)[0]
   neighbors = list(G.neighbors(node))
   if len(neighbors) > 0:
      avg_neigh_polarity = sum(G.nodes[neigh]["polarity"] for neigh in neighbors) / len(neighbors)
   else:
      avg_neigh_polarity = random.random()
   polarity_differences = np.abs(_user_polarity - avg_neigh_polarity)
   idx = polarity_differences.argsort()
   G.nodes[node]["polarity"] = _user_polarity[idx[0]]
   polarity_user[_user_polarity[idx[0]]] = node
   _user_polarity = np.delete(_user_polarity, idx[0])
   users_needing_polarity.remove(node)

with open("user_polarity", 'w') as f:
   for n in G.nodes:
      f.write("%s\t%s\n" % (n, G.nodes[n]["polarity"]))

with open("user_polarity_homophily", 'w') as f:
   x = []
   y = []
   for n in G.nodes:
      for n2 in G.neighbors(n):
         f.write("%s\t%s\n" % (G.nodes[n]["polarity"], G.nodes[n2]["polarity"]))
         x.append(G.nodes[n]["polarity"])
         y.append(G.nodes[n2]["polarity"])
   print("User polarity homophily: %1.4f (%1.4f)" % pearsonr(x, y))

# Assign users to news sources. Each news source gets its popularity worth of users,
# and we pick them so that they're the closest possible in polarity
G_source_user = nx.Graph()
user_nsources = defaultdict(int)
for i in range(len(source_popularity)):
   polarity_differences = np.abs(user_polarity - source_polarity[i])
   idx = polarity_differences.argsort()
   audience_polarities = user_polarity[idx < source_popularity[i]]
   for audience_polarity in audience_polarities:
      G_source_user.add_edge("s%s" % i, "u%s" % polarity_user[audience_polarity])
      user_nsources[polarity_user[audience_polarity]] += 1

nx.write_edgelist(G_source_user, "audience_network", data = False)
with open("user_nsources", 'w') as f:
   for n in G.nodes:
      f.write("%s\t%s\n" % (n, user_nsources[n]))

print("avg sources per user: %1.2f" % (sum(user_nsources.values()) / len(user_nsources)))
os.chdir("../..")
